#!/bin/sh

ACTION=${1}
shift 1

get_image() {
  IMAGE_NAME=$(uci get autogpt.@autogpt[0].image_name 2>/dev/null)
}

do_install_detail() {
  local openai=`uci get autogpt.@autogpt[0].openai 2>/dev/null`
  local config_path=`uci get autogpt.@autogpt[0].config_path 2>/dev/null`
  local image_name=`uci get autogpt.@autogpt[0].image_name 2>/dev/null`
  local mem_back=`uci get autogpt.@autogpt[0].mem_back 2>/dev/null`
  local mem_index=`uci get autogpt.@autogpt[0].mem_index 2>/dev/null`
  local headless=`uci get autogpt.@autogpt[0].headless 2>/dev/null`
  local execute=`uci get autogpt.@autogpt[0].execute 2>/dev/null`
  local chunk_max=`uci get autogpt.@autogpt[0].chunk_max 2>/dev/null`
  local smart_llm=`uci get autogpt.@autogpt[0].smart_llm 2>/dev/null`
  local fast_llm=`uci get autogpt.@autogpt[0].fast_llm 2>/dev/null`
  local fast_token=`uci get autogpt.@autogpt[0].fast_token 2>/dev/null`
  local smart_token=`uci get autogpt.@autogpt[0].smart_token 2>/dev/null`
  local port=`uci get autogpt.@autogpt[0].port 2>/dev/null`

  GEN_PASS=$(< /dev/urandom tr -dc A-Za-z0-9 2>/dev/null | head -c14; echo)
  GEN_PASS2=$(< /dev/urandom tr -dc A-Za-z0-9 2>/dev/null | head -c14; echo)

  LAN_IP=$(uci get network.lan.ipaddr)
  LAN_IP="${LAN_IP%/*}"

  if [ -z "$config" ]; then
    echo "config path is empty!"
    exit 1
  fi

  [ -z "$port" ] && port={port}
  [ -z "$IMAGE_NAME" ] && IMAGE_NAME={image}

  rm -R /opt/docker2/compose/autogpt
  mkdir /opt/docker2/compose/autogpt

  touch /opt/docker2/compose/autogpt/docker-compose.yml
  cat > /opt/docker2/compose/autogpt/docker-compose.yml <<EOF
version: "3.9"

services:
  auto-gpt:
    image: $image_name
    restart: on-failure
    init: true
    stop_grace_period: 1m
    tty: true
    environment:
      - OPENAI_API_KEY=$openai
      - MEMORY_BACKEND=$mem_back
      - MEMORY_INDEX=$mem_index
      - HEADLESS_BROWSER=$headless
      - EXECUTE_LOCAL_COMMANDS=$execute 
      - BROWSE_CHUNK_MAX_LENGTH=$chunk_max
      - SMART_LLM_MODEL=$smart_llm
      - FAST_LLM_MODEL=$fast_llm
      - FAST_TOKEN_LIMIT=$fast_token
      - SMART_TOKEN_LIMIT=$smart_token
    volumes:
      - $config_path:/app/auto_gpt_workspace:rw
      - ./data:/app/data:rw
      - ./logs:/app/logs:rw
    ports:
      - $port:3000

networks:
  default:
    name: auto-gpt
EOF

docker compose build auto-gpt
docker compose run --rm auto-gpt

  uci add shortcutmenu lists
  uci set shortcutmenu.@lists[-1].webname="$IMAGE_NAME"
  uci set shortcutmenu.@lists[-1].weburl="$LAN_IP:$port"
  uci set shortcutmenu.@lists[-1].webpath="/"
  uci commit shortcutmenu
}

usage() {
  echo "usage: $0 sub-command"
  echo "where sub-command is one of:"
  echo "      install                Install autogpt"
  echo "      upgrade                Upgrade autogpt"
  echo "      rm/start/stop/restart  Remove/Start/Stop/Restart autogpt"
  echo "      status                 autogpt status"
  echo "      port                   autogpt port"
}

case ${ACTION} in
  "install"|"upgrade")
    get_image
    do_install_detail
  ;;
  "rm")
    get_image
    CONTAINER_IDS=$(docker ps -a --filter "ancestor=${IMAGE_NAME}" --format '{{.ID}}')
    echo "Stopping and removing containers..."
    for ID in $CONTAINER_IDS; do
      docker stop $ID
      docker rm $ID
    done
    docker rmi -f $IMAGE_NAME
  ;;
  "start"|"stop"|"restart")
    get_image
    CONTAINER_IDS=$(docker ps -a --filter "ancestor=${IMAGE_NAME}" --format '{{.ID}}')
    for ID in $CONTAINER_IDS; do
      docker ${ACTION} ${ID}
    done
  ;;
  "status")
    get_image
    CONTAINER_NAMES=$(docker ps -a --filter "ancestor=${IMAGE_NAME}" --format '{{.Names}}')
    docker ps --all -f "name=${CONTAINER_NAMES}" --format '{{.Status}}'
  ;;
  "port")
    get_image
    CONTAINER_NAMES=$(docker ps -a --filter "ancestor=${IMAGE_NAME}" --format '{{.Names}}')
    docker ps --all -f "name=${CONTAINER_NAMES}" --format '{{.Ports}}' | grep -om1 '0.0.0.0:[0-9]*' | sed 's/0.0.0.0://'
  ;;
  *)
    usage
    exit 1
  ;;
esac